package	jp.co.sds_corp.nanchart.graph;

import java.awt.BasicStroke;
import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.Stroke;
import java.awt.image.BufferedImage;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Vector;

/**
GraphSeriesは、全てのグラフの抽象基底クラスです。
<BR>
各サブクラスによって実装されなければならない重要なメソッドは、グラフを描画する
{@link #drawPoint(Graphics2D,Rectangle,Object,Object,Object,Object,Object,Object,Object)}メソッドです。
<BR>
グラフは{@link GraphStorage}クラスにデータを格納します。デフォルトでは
{@link DefaultStorage}を使用するよう初期化されます。{@link DefaultStorage}以外の
クラスにデータを格納する必要がある場合は、{@link #setStorage(GraphStorage)}で
データ格納クラスを変更してください。
<BR><BR>
X軸・Y軸の範囲は、Graph2Dの範囲と必ずしも一致する必要はありません。
@see	AreaGraph
@see	BarGraph
@see	LineGraph
@see	PieGraph
@see	RosokuGraph
@see	ScatterGraph
*/
public abstract class GraphSeries
{
	/** 描画に使用する線の色 */
	Color			LineCol = Color.black;
	/** 描画に使用する面の色 */
	Color			FillCol = null; //Color.white;
	/** 描画に使用するペン先の形 */
	Stroke			str = new BasicStroke( 2 );
	/** 補助線の描画に使用するペン先の形 */
	Stroke	substr = null;
	/** 描画に使用するフォント */
	Font			font;
	/** この系列に関係付けられた軸 */
	Vector<GraphScale>		vecScale = new Vector<GraphScale>	();
	/** データ格納クラス */
	GraphStorage	store;
	/*** グラフ全体を表すGraph2Dオブジェクト */
	Graph2D			parent = null;
	/** 系列ごとの色を保存します */
	Hashtable	hashFillCol = new Hashtable();
	/** 系列ごとの色を保存します */
	Hashtable	hashLineCol = new Hashtable();
	boolean		bArea = false;
	boolean		bStack = false;

	/**
	* デフォルトの状態でGraphSeriesを構築します。
	*/
	public GraphSeries() {
		setStorage( new DefaultStorage() );
	}

	/**
	* X軸・Y軸の範囲を指定して、でGraphSeriesを構築します。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public GraphSeries( double xMin, double xMax, double yMin, double yMax ) {
		setStorage( new DefaultStorage( xMin, xMax, yMin, yMax ) );
	}

	/**
	* X軸・Y軸の範囲を指定して、でGraphSeriesを構築します。
	* 指定するオブジェクトのタイプについては、{@link Value}クラスの説明を
	* 参照してください。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public GraphSeries( Object xMin, Object xMax, Object yMin, Object yMax ) {
		setStorage(  new DefaultStorage( xMin, xMax, yMin, yMax ) );
	}

	/**
	* データ格納クラスを指定して、でGraphSeriesを構築します。
	* @param	map		データ格納クラス
	*/
	public GraphSeries( GraphStorage map ) {
		setStorage( map );
	}

	/**
	* X軸の範囲を設定します。
	* 設定するオブジェクトのタイプについては、{@link Value}クラスの説明を
	* 参照してください。
	* @param	xMin	X軸の最小値
	* @param	xMax	X軸の最大値
	*/
	public void setXRange( Object xMin, Object xMax ) {
		store.setXRange( xMin, xMax );
	}

	/**
	* Y軸の範囲を設定します。
	* 設定するオブジェクトのタイプについては、{@link Value}クラスの説明を
	* 参照してください。
	* @param	yMin	Y軸の最小値
	* @param	yMax	Y軸の最大値
	*/
	public void setYRange( Object yMin, Object yMax ) {
		store.setYRange( yMin, yMax );
	}

	/**
	* 描画に使用する線の色を設定します。
	* @param	col		線の色
	*/
	public void setLineColor( Color col ) {
		this.LineCol = col;
	}
	/**
	* 系列ごとの色を設定します。
	* @param	srs		系列を表すオブジェクト
	* @param	col		塗りつぶす色
	*/
	public void setLineColor( Object srs, Color col ) {
		hashLineCol.put( srs, col );
	}

	/**
	* 塗りつぶしに使用する色を設定します。
	* @param	col		塗りつぶしに使用する色
	*/
	public void setFillColor( Color col ) {
		this.FillCol = col;
	}
	/**
	* 系列ごとの色を設定します。
	* @param	srs		系列を表すオブジェクト
	* @param	col		塗りつぶす色
	*/
	public void setFillColor( Object srs, Color col ) {
		hashFillCol.put( srs, col );
	}

	/**
	* 描画に使用するペン先を設定します。
	* @param	str		描画するのに使われる Stroke オブジェクト
	*/
	public void setStroke( Stroke str ) {
		this.str = str;
	}
	/**
	* 描画に使用するペン先を取得します。
	* @return	描画するのに使われる Stroke オブジェクト
	*/
	public Stroke getStroke() {
		return str;
	}

	/**
	* 補助線の描画に使用するペン先を設定します。
	* @param	str		描画するのに使われる Stroke オブジェクト
	*/
	public void setSubStroke( Stroke str ) {
		this.substr = str;
	}
	/**
	* 補助線の描画に使用するペン先を取得します。
	* @return	描画するのに使われる Stroke オブジェクト
	*/
	public Stroke getSubStroke() {
		return substr;
	}

	/**
	* 描画に使用するフォントを設定します。
	* @param	font	フォント
	*/
	public void setFont( Font font ) {
		this.font = font;
	}
	/**
	* 描画に使用するフォントを取得します。
	* @return	フォント
	*/
	public Font getFont() {
		return font;
	}

	/**
	* 描画に使用するデータ格納クラスを取得します。
	* @return	データ格納クラス
	*/
	public GraphStorage getStorage() {
		return store;
	}
	/**
	* 描画に使用するデータ格納クラスを設定します。
	* @param	store	データ格納クラス
	*/
	public void setStorage( GraphStorage store ) {
		this.store = store;
	}

	/**
	* グラフ全体を表すGraph2Dオブジェクトを返します。
	* @return	グラフ全体
	*/
	public Graph2D getParent() {
		return parent;
	}
	/**
	* グラフ全体を表すGraph2Dオブジェクトを設定します。
	* @param	parent	グラフ全体
	*/
	public void setParent( Graph2D parent ) {
		this.parent = parent;
	}

	/**
	* イメージのうち、指定された領域を切り出します。
	* @param	img		元のイメージ
	* @param	rect	切り出す矩形
	*/
	public BufferedImage getSubimage( BufferedImage img, Rectangle rect ) {
		return img.getSubimage( (int )rect.getX(), (int )rect.getY(),
								(int )rect.getWidth(), (int )rect.getHeight());
	}

	/**
	* この系列に関係付けられた軸を追加します。
	* @param	scale	軸
	*/
	public void addScale( GraphScale scale ) {
		vecScale.add( scale );
	}

	/**
	* この系列に関係付けられた軸を取得します。
	* @return	軸
	*/
	public java.util.List listScale() {
		return vecScale;
	}

	/**
	* 目盛り線をイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScaleLine( Graphics2D gr, Rectangle rect ) {
		for ( int i = 0; i < vecScale.size(); i++ ) {
			GraphScale scale = (GraphScale )vecScale.get( i );
			scale.drawScaleLine( gr, rect );
		}
	}

	/**
	* 目盛りをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawScale( Graphics2D gr, Rectangle rect ) {
		for ( int i = 0; i < vecScale.size(); i++ ) {
			GraphScale scale = (GraphScale )vecScale.get( i );
			scale.drawScale( gr, rect );
		}
	}

	/**
	* グラフをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	イメージ中のグラフの領域
	*/
	public void drawGraph( Graphics2D gr, Rectangle rect ) {
		if ( store.getXMax() == null ) {
			store.setXRange();
		}
		if ( store.getYMax() == null ) {
			store.setYRange();
		}

		gr.setStroke( str );

		Object		srs;
		Iterator	itr;
		Iterator	itrSrs;
		Map			mapKey;
		Map			mapVal;
		Object		topX = null;
		Object		topY1 = null, topY2 = null;
		Object		curX = null;
		Object		curY1 = null, curY2 = null;
		Object		preX = null;
		Object		preY1 = null, preY2 = null;
		DefaultStorage	stack = (DefaultStorage )store;

		itrSrs = store.getIterator();
		while ( itrSrs.hasNext() ) {
			srs = itrSrs.next();
			mapVal = stack.getMapVal( srs );
			mapKey = stack.getKeys();
			itr = mapKey.keySet().iterator();
			while ( itr.hasNext() ) {
				curX = itr.next();
				curY1 = mapKey.get( curX );

				if ( bStack ) {
					stack.stack( curX, mapVal.get( curX ) );
					curY2 = mapKey.get( curX );
				} else {
					curY2 = mapVal.get( curX );
				}

				if ( topX == null ) {
					topX = curX;
					topY1 = curY1;
					topY2 = curY2;
				}

				drawPoint( gr, rect, srs, curX, curY1, curY2, preX, preY1, preY2 );

				preX = curX;
				preY1 = curY1;
				preY2 = curY2;
			}
			endSeries( gr, rect, srs, curX, curY1, curY2, topX, topY1, topY2 );
			preX = topX = null;
		}
	}

	/**
	* グラフ描画の後処理を行ないます。
	* @param	gr		描画するデバイス
	* @param	rect	描画する矩形
	* @param	srs		描画するグラフ
	* @param	curX	現在のX座標
	* @param	curY1	現在のY座標1
	* @param	curY2	現在のY座標2
	* @param	preX	前回のX座標
	* @param	preY1	前回のY座標1
	* @param	preY2	前回のY座標2
	*/
	public void endSeries( Graphics2D gr, Rectangle rect, Object srs,
				Object curX, Object curY1, Object curY2,
				Object preX, Object preY1, Object preY2 )
	{
	}

	/**
	* グラフのデータをイメージに出力します。
	* @param	gr		出力先
	* @param	rect	出力先の領域
	* @param	srs		描画するグラフ
	* @param	cX		現在のX座標
	* @param	cY1		現在のY座標1
	* @param	cY2		現在のY座標2
	* @param	pX		前回のX座標
	* @param	pY1		前回のY座標1
	* @param	pY2		前回のY座標2
	*/
	public abstract void drawPoint( Graphics2D gr, Rectangle rect,
				Object srs,
				Object cX, Object cY1, Object cY2,
				Object pX, Object pY1, Object pY2 );

	/**
	* 全体の値から算出したX軸の値を算出します。
	* @param	rect	グラフを描画する矩形
	* @param	x		X座標のオブジェクト
	* @param	y		Y座標のオブジェクト
	*/
	public double getXPos( Rectangle rect, Object x, Object y ) {
		return store.getXPos( rect, x, y );
	}

	/**
	* 全体の値から算出したY軸の値を算出します。
	* @param	rect	グラフを描画する矩形
	* @param	x		X座標のオブジェクト
	* @param	y		Y座標のオブジェクト
	*/
	public double getYPos( Rectangle rect, Object x, Object y ) {
		return store.getYPos( rect, x, y );
	}

}

